/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.model.IRemovable;
import org.fortiss.tooling.kernel.services.IActionService;
import org.fortiss.tooling.kernel.services.ICommandStackService;
import org.fortiss.tooling.kernel.util.EObjectSelectionUtils;

/**
 * This class implements the {@link IActionService} interface.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class ActionService implements IActionService {

	/** The group id of the global default action group. */
	public static final String GROUP_GLOBAL_DEFAULTS = "globalDefaults";

	/** The global undo action. */
	public final IAction globalUndoAction = new Action("Undo", PlatformUI
			.getWorkbench().getSharedImages()
			.getImageDescriptor(ISharedImages.IMG_TOOL_UNDO)) {

		@Override
		public void run() {
			EObject selectedObject = EObjectSelectionUtils
					.getCurrentSelectionFirstElement();
			if (selectedObject != null) {
				ICommandStackService.INSTANCE.undo(selectedObject);
			}
		}

		@Override
		public boolean isEnabled() {
			EObject selectedObject = EObjectSelectionUtils
					.getCurrentSelectionFirstElement();
			return selectedObject != null
					&& ICommandStackService.INSTANCE.canUndo(selectedObject);
		}
	};

	/** The global delete action. */
	public final IAction globalDeleteAction = new Action("Delete", PlatformUI
			.getWorkbench().getSharedImages()
			.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE)) {
		@Override
		public void run() {
			final EObject selectedObject = EObjectSelectionUtils
					.getCurrentSelectionFirstElement();
			ICommandStackService.INSTANCE.runAsCommand(selectedObject,
					new Runnable() {
						@Override
						public void run() {
							if (selectedObject != null) {
								if (selectedObject instanceof IRemovable) {
									((IRemovable) selectedObject).remove();
								} else {
									EcoreUtil.delete(selectedObject, true);
								}
							}
						}
					});
		}

		@Override
		public boolean isEnabled() {
			EObject selectedObject = EObjectSelectionUtils
					.getCurrentSelectionFirstElement();
			if (selectedObject != null) {
				if (selectedObject instanceof IRemovable) {
					return ((IRemovable) selectedObject).canRemove();
				}
				return selectedObject.eContainer() != null;
			}
			return false;
		}
	};

	/** {@inheritDoc} */
	@Override
	public void addGlobalDefaultActionSectionToMenu(MenuManager menuManager) {
		Separator globalDefaults = new Separator(GROUP_GLOBAL_DEFAULTS);
		globalDefaults.setVisible(true);
		menuManager.add(globalDefaults);

		menuManager.appendToGroup(GROUP_GLOBAL_DEFAULTS, globalDeleteAction);
		menuManager.appendToGroup(GROUP_GLOBAL_DEFAULTS, globalUndoAction);
	}

	/** {@inheritDoc} */
	@Override
	public void registerGlobalActions(IViewSite site) {
		fillActionBars(site.getActionBars());
	}

	/** {@inheritDoc} */
	@Override
	public void registerGlobalActions(IEditorSite site) {
		fillActionBars(site.getActionBars());
	}

	/** Fills the given action bar with the global actions. */
	private void fillActionBars(IActionBars actionBars) {
		globalUndoAction.setId(ActionFactory.UNDO.getId());
		globalUndoAction.setActionDefinitionId(ActionFactory.UNDO
				.getCommandId());
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
				globalUndoAction);

		actionBars.updateActionBars();
	}

	/** {@inheritDoc} */
	@Override
	public void update() {
		globalUndoAction.setEnabled(globalUndoAction.isEnabled());
	}
}
